Package: mpra
Version: 1.34.0
Title: Analyze massively parallel reporter assays
Description: Tools for data management, count preprocessing, and differential analysis in massively parallel report assays (MPRA).
Authors@R: c(
	   person("Leslie", "Myint", role = c("cre", "aut"), email = "leslie.myint@gmail.com"),
	   person(c("Kasper", "D."), "Hansen", role = "aut"))
Depends: R (>= 3.5.0), methods, BiocGenerics, SummarizedExperiment,
        limma
Suggests: BiocStyle, knitr, rmarkdown, RUnit
Imports: S4Vectors, scales, stats, graphics, statmod
Collate: mpra_set.R utils.R fit.R
VignetteBuilder: knitr
License: Artistic-2.0
URL: https://github.com/hansenlab/mpra
BugReports: https://github.com/hansenlab/mpra/issues
biocViews: Software, GeneRegulation, Sequencing, FunctionalGenomics
git_url: https://git.bioconductor.org/packages/mpra
git_branch: RELEASE_3_23
git_last_commit: 03a55cf
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:54:16 UTC; biocbuild
Author: Leslie Myint [cre, aut],
  Kasper D. Hansen [aut]
Maintainer: Leslie Myint <leslie.myint@gmail.com>
Built: R 4.6.0; ; 2026-04-29 04:51:35 UTC; unix
