import(methods)
import(BiocGenerics, except = "plotMA")
import(SummarizedExperiment)
import(limma, except = "plotMA")
import(S4Vectors)
importFrom("scales", "alpha")
importFrom("graphics", "lines")
importFrom("stats", "approxfun", "lowess")
importFrom("statmod", "mixedModel2Fit")

exportClasses("MPRASet")
exportMethods("show")
export(MPRASet, getRNA, getDNA, getBarcode, getEid, getEseq)
export(mpralm, get_precision_weights, compute_logratio, normalize_counts)
